#include "induct.h"
/*=================================================
       evaluation of the definite integral of 
         2-D Green's function
***************************************************/

/* This is the integral of Green's function (without the constant).  Since
   we need the definite integral to get Pnm, we use it in the function p (many times) */

double F(double e, double n)
{
 double p,e2,n2,t,x,y;

/* atan is always bounded (see below), so if e,n = 0 y -> 0 and x -> 0 so return 0*/
/*
    The domain of tan (range of atan) must be treated with care...
    atan2 will give -pi to pi, atan gives -pi/2 to pi/2.
 it seems -pi/2 to pi/2 is correct
*/

 e2 = e * e;
 n2 = n * n;

 if(fabs(e) <= SMALL)
   if(fabs(n) <= SMALL)
     return(0.0);
   else
     return(log(n2)*n2*n2);
 else if(fabs(n) <= SMALL)
   return(log(e2)*e2*e2);
 else
 {
   p = e * n;
   t = atan(e/n);
   x = log(e2 + n2) * (e2*e2 - 6*p*p + n2*n2) + 25*p*p;
   if (p > 0)
      return(x - 8*p*(e2*(hPI - t) + n2*t));
    else
      return(x - 8*p*(e2*(-hPI - t) + n2*t));
 }
} /* F */
/* The definite integral of Green's function (with constant current density) */


